# tinyrox says don't edit this manually, but it can't stop you!

export(active_media_reference_key)
export("active_media_reference_key<-")
export(add_child)
export(add_effect)
export(add_track)
export(almost_equal)
export(append_child)
export(audio_tracks)
export(available_range)
export("available_range<-")
export(children)
export(clamped)
export(clear_children)
export(Clip)
export(clone)
export(color)
export("color<-")
export(comment)
export("comment<-")
export(contains)
export(default_media_key)
export(duration)
export(Effect)
export(effect_name)
export("effect_name<-")
export(effects)
export(enabled)
export("enabled<-")
export(end_frame)
export(end_time_exclusive)
export(end_time_inclusive)
export(extended_by)
export(ExternalReference)
export(fill)
export(find_clips)
export(flatten_stack)
export(frame_for_time)
export(frame_step)
export(frame_zero_padding)
export(FreezeFrame)
export(from_frames)
export(from_json_file)
export(from_json_string)
export(from_seconds)
export(from_time_string)
export(from_timecode)
export(Gap)
export(generator_kind)
export("generator_kind<-")
export(GeneratorReference)
export(global_start_time)
export("global_start_time<-")
export(has_child)
export(has_clips)
export(ImageSequenceReference)
export(in_offset)
export("in_offset<-")
export(index_of_child)
export(insert)
export(insert_child)
export(intersects)
export(is_composition)
export(is_effect)
export(is_equivalent_to)
export(is_media_reference)
export(is_missing_reference)
export(is_otio)
export(is_parent_of)
export(is_rational_time)
export(is_time_range)
export(is_timeline)
export(is_unknown_schema)
export(Item)
export(kind)
export("kind<-")
export(LinearTimeWarp)
export(marked_range)
export("marked_range<-")
export(Marker)
export(media_reference)
export("media_reference<-")
export(media_references)
export(MediaReference)
export(metadata)
export("metadata<-")
export(MissingReference)
export(name)
export(name_prefix)
export(name_suffix)
export("name<-")
export(number_of_images_in_sequence)
export(out_offset)
export("out_offset<-")
export(overlapping)
export(overlaps)
export(overwrite)
export(parameters)
export("parameters<-")
export(parent)
export(presentation_time_for_image_number)
export(range_from_start_end_time)
export(range_in_parent)
export(rate)
export(RationalTime)
export(read_otiod)
export(register_downgrade_function)
export(register_upgrade_function)
export(remove)
export(remove_child)
export(rescaled_to)
export(ripple)
export(roll)
export(schema_name)
export(schema_version)
export(SerializableCollection)
export(set_child)
export(set_children)
export("set_children<-")
export(set_media_references)
export(slice)
export(slide)
export(slip)
export(source_range)
export("source_range<-")
export(Stack)
export(start_frame)
export(start_time)
export(target_url)
export(target_url_base)
export("target_url_base<-")
export(target_url_for_image_number)
export("target_url<-")
export(time_scalar)
export("time_scalar<-")
export(TimeEffect)
export(Timeline)
export(TimeRange)
export(TimeTransform)
export(to_frames)
export(to_json_file)
export(to_json_string)
export(to_seconds)
export(to_time_string)
export(to_timecode)
export(Track)
export(track_trimmed_to_range)
export(tracks)
export("tracks<-")
export(Transition)
export(transition_type)
export("transition_type<-")
export(trim)
export(trimmed_range)
export(trimmed_range_in_parent)
export(type_version_map)
export(validate_with_RcppOTIO)
export(value)
export(video_tracks)
export(visible)
export(visible_range)
export(write_otiod)

S3method(print,Clip)
S3method(print,Composition)
S3method(print,ExternalReference)
S3method(print,RationalTime)
S3method(print,Timeline)
S3method(print,TimeRange)
